# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from scarfer.scan_interface import ScanReportReader
from scarfer.analyzer import ScanReportFilter
from scarfer.analyzer import ScanReportFilterType
from scarfer.analyzer import Analyzer

CAIRO_REPORT_0 = "example-data/scancode/no-format/cairo-1.16.0-scan.json"
CAIRO_REPORT_1 = "example-data/scancode/1.0.0/cairo-1.16.0-scan.json"
CAIRO_REPORT_2 = "example-data/scancode/2.0.0/cairo-1.16.0-scan.json"

class TestScancodeReader_0(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestScancodeReader_0, self).__init__(*args, **kwargs)
        self._setup(CAIRO_REPORT_0)

    def _setup(self, report):
        self.reader = ScanReportReader(report)
        normalized_report = self.reader.read()
        self.analyzer = Analyzer(normalized_report)
        self.analyzer.apply_filters()
        self.data = self.analyzer.report()
        self.before_count = len(self.data['files'])
        self.assertEqual(self.before_count, 4686)

    def test_license_filter(self):
        # filter in files with license with "x11"
        license_filter = ScanReportFilter("x11", ScanReportFilterType.LICENSE)
        # apply filter
        self.analyzer.apply_filters([license_filter])

        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])
        self.assertEqual(after_count, 309)

    def test_copyright_filter(self):
        # filter in files with copyright with "laar"
        copyright_filter = ScanReportFilter("laar", ScanReportFilterType.COPYRIGHT)
        # apply filter
        self.analyzer.apply_filters([copyright_filter])

        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])
        self.assertEqual(after_count, 9)

    def test_file_filter(self):

        # filter in files with path with "cairo-xcb"
        file_filter = ScanReportFilter("cairo-xcb")

        # apply filter
        self.analyzer.apply_filters([file_filter])
        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])
        self.assertEqual(after_count, 13)

    def test_file_license_filters(self):

        # filter in files with license with "x11"
        license_filter = ScanReportFilter("x11", ScanReportFilterType.LICENSE)
        # filter in files with path with "cairo-xcb"
        file_filter = ScanReportFilter("cairo-xcb")

        # apply filter
        self.analyzer.apply_filters([file_filter, license_filter])
        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])
        self.assertEqual(after_count, 1)

    def test_file_copyright_filters(self):

        # filter in files with copyright with "laar" (9 of them)
        copyright_filter = ScanReportFilter("laar", ScanReportFilterType.COPYRIGHT)
        # filter out files with path with "test" (6 of the 9)
        file_filter = ScanReportFilter("test")

        # apply filter
        self.analyzer.apply_filters([copyright_filter], [file_filter])

        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])
        self.assertEqual(after_count, 3)

    def test_file_exclude(self):

        # filter out files with license with "x11"
        license_filter = ScanReportFilter("x11-keith", ScanReportFilterType.LICENSE)

        # apply filter
        self.analyzer.apply_filters([], [license_filter])
        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])

        # $ PYTHONPATH=. ./scarfer/__main__.py example-data/cairo-1.16.0-scan.json -l | grep -v x11-keith  | grep ^cairo | wc -l
        # 4378
        self.assertEqual(after_count, 4378)

    def test_license_exclude(self):

        # filter out files with path with "test"
        file_filter = ScanReportFilter("/test")

        # apply filter
        self.analyzer.apply_filters([], [file_filter])
        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])

        # $ PYTHONPATH=. ./scarfer/__main__.py example-data/cairo-1.16.0-scan.json -l | grep -v /test | grep ^cairo | wc -l
        # 745
        self.assertEqual(after_count, 745)

    def test_file_license_exclude(self):

        # filter out files with license with "x11"
        license_filter = ScanReportFilter("x11-keith", ScanReportFilterType.LICENSE)

        # filter out files with path with "test"
        file_filter = ScanReportFilter("/test")

        # apply filter
        self.analyzer.apply_filters([], [file_filter, license_filter])
        filtered_data = self.analyzer.report()

        self.assertIsNotNone(filtered_data)
        after_count = len(filtered_data['files'])

        # $ PYTHONPATH=. ./scarfer/__main__.py example-data/cairo-1.16.0-scan.json -l | grep -v /test | grep -v x11-keith  | grep ^cairo | wc -l
        # 683
        self.assertEqual(after_count, 683)

class TestScancodeReader_1(TestScancodeReader_0):

    def __init__(self, *args, **kwargs):
        super(TestScancodeReader_1, self).__init__(*args, **kwargs)
        self._setup(CAIRO_REPORT_1)

class TestScancodeReader_2(TestScancodeReader_0):

    def __init__(self, *args, **kwargs):
        super(TestScancodeReader_2, self).__init__(*args, **kwargs)
        self._setup(CAIRO_REPORT_2)


if __name__ == '__main__':
    unittest.main()



# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from scarfer.format.factory import FormatFactory
from scarfer.scan_interface import ScanReportReader

CAIRO_REPORT = "example-data/scancode/no-format/cairo-1.16.0-scan.json"

class TestScancodeReader(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestScancodeReader, self).__init__(*args, **kwargs)

    def test_reader(self):
        reader = ScanReportReader(CAIRO_REPORT)
        self.assertIsNotNone(reader)

    def test_formatter(self):
        formatter = FormatFactory.formatter("json")
        self.assertIsNotNone(formatter)
        formatter = FormatFactory.formatter("JSON")
        self.assertIsNotNone(formatter)


if __name__ == '__main__':
    unittest.main()



# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from scarfer.scan_interface import ScanReportReader
from scarfer.analyzer import Analyzer

CAIRO_REPORT_2 = "example-data/scancode/2.0.0/cairo-1.16.0-scan.json"
CAIRO_REPORT_3 = "example-data/scancode/3.0.0/cairo-1.16.0-scan.json"

class TestScancodeReader(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestScancodeReader, self).__init__(*args, **kwargs)

    def test_missing_license_2_0_0(self):
        """
        File cairo-1.16.0/util/show-events.c has no license
        make sure this is analyzed as  ['missing']
        In 2.0.0, scancode says: []
        """
        reader = ScanReportReader(CAIRO_REPORT_2)
        report = reader.read()
        analyzer = Analyzer(report)
        analyzer.apply_filters()
        analyzed_report = analyzer.report()
        for _file in analyzed_report['files']:
            if _file['path'] == 'cairo-1.16.0/util/show-events.c':
                _file_lic = _file['license']['expressions']
                self.assertEqual(_file_lic, ['missing'])

    def test_missing_license_3_0_0(self):
        """
        File cairo-1.16.0/util/show-events.c has no license
        make sure this is analyzed as  ['missing']
        In 3.0.0, scancode says: none
        """
        reader = ScanReportReader(CAIRO_REPORT_3)
        report = reader.read()
        analyzer = Analyzer(report)
        analyzer.apply_filters()
        analyzed_report = analyzer.report()
        for _file in analyzed_report['files']:
            if _file['path'] == 'cairo-1.16.0/util/show-events.c':
                _file_lic = _file['license']['expressions']
                self.assertEqual(_file_lic, ['missing'])

    def test_matching_license_2_0_0(self):
        """
        File cairo-1.16.0/util/cairo-script/cairo-script-file.c has two licenses identified
        """
        reader = ScanReportReader(CAIRO_REPORT_2)
        report = reader.read()
        analyzer = Analyzer(report)
        analyzer.apply_filters()
        analyzed_report = analyzer.report()
        for _file in analyzed_report['files']:
            if _file['path'] == 'cairo-1.16.0/util/cairo-script/cairo-script-file.c':
                match_count = len(_file['license']['matches'])
                self.assertEqual(2, match_count)

                file_lic = _file['license']['expressions']
                self.assertEqual(['lgpl-2.1 OR mpl-1.1'], file_lic)

    def test_matching_license_3_0_0(self):
        """
        File cairo-1.16.0/util/cairo-script/cairo-script-file.c has THREE licenses identified
        """
        reader = ScanReportReader(CAIRO_REPORT_3)
        report = reader.read()
        analyzer = Analyzer(report)
        analyzer.apply_filters()
        analyzed_report = analyzer.report()
        for _file in analyzed_report['files']:
            if _file['path'] == 'cairo-1.16.0/util/cairo-script/cairo-script-file.c':
                match_count = len(_file['license']['matches'])
                self.assertEqual(3, match_count)

                file_lic = _file['license']['expressions']
                # THIS EXPRESSION SHOULD BE WITH AN "AND" Until
                # https://github.com/nexB/scancode-toolkit/issues/3523
                # is fixed, we need to keep it this way When the
                # scancode bug is fixed, rescan and replace current
                # scan report
                self.assertEqual(['lgpl-2.1 AND mpl-1.1'], file_lic)



# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import json
import jsonschema
import os

SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))


# expected result from read(), from the implementing classes
# ----------------------
# files: [
#   {
#     "path":       "tmp/tmp/apa.c",
#     "copyrights": [ "(c) 2009 Some One", "(c) 2001 Someone Elsie" ],
#     "license":    {
#       "expression": [ "mit or x11", "bsd" ],
#       "matches": [
#         {
#           "key":  "mit or x11",
#           "text": "This library is license under mit or x11"
#         }
#       ]
#     }
#  ]

#
# what to expect from report() - as a user
#
# {
#   "files": (see above)
#   "meta": {}
#   "cumulative": {
#     "license": "simplified license of all the combined"
#     "copytights": "combined copyrights"
#   }


class ScanReportReader:

    def __init__(self, file_name):
        self.file_name = file_name
        self.report_data = None
        self.schema = None

    def report_file(self):
        return self.file_name

    def validate(self):
        if not self.schema:
            schema_file = os.path.join(os.path.join(SCRIPT_DIR, "var"), "normalized-scan.json")
            with open(schema_file, 'r') as f:
                self.schema = json.load(f)
            jsonschema.validate(instance=self.data, schema=self.schema)

    def read(self):
        clazzes = [FakeReportReader, ScancodeReportReader]
        self.data = None
        for clazz in clazzes:
            exceptions = []
            try:
                self.data = clazz(self.file_name).read()
                break
            except FileNotFoundError:
                raise (ScanReportException(f'File {self.file_name} not found'))
            except Exception as e:
                exceptions.append(e)
        if self.data is None:
            raise (ScanReportException(f'File {self.file_name} not in a supported format. Exceptions caught: {exceptions}'))

        return self.data

    def __str__(self):
        ret = ["path: {name}".format(name=self.file_name)]
        return '\n'.join(ret)

class ScanReportException(Exception):

    def __init__(self, message=""):
        self.message = message
        super().__init__(self.message)

class FakeReportReader(ScanReportReader):

    def read(self):
        raise (ScanReportException("File not in fake format"))

class ScancodeReportReader(ScanReportReader):

    def check_file_format(self):
        headers = self.json_data['headers'][0]
        tool = headers['tool_name']
        self.scancode_format = headers.get('output_format_version', '')
        if tool.lower() != "scancode-toolkit":
            raise (ScanReportException(f'File not in Scancode format. Tool={tool}'))

        if self.scancode_format == "2.0.0":
            self.copyright_value = "copyright"
            self.licenses_value = "licenses"
        elif self.scancode_format[:3] == "3.0":
            self.copyright_value = "copyright"
            self.licenses_value = "license_detections"
        elif self.scancode_format[:3] == "3.2":
            self.copyright_value = "copyright"
            self.licenses_value = "license_detections"
        elif self.scancode_format[:3] == "4.0":
            self.copyright_value = "copyright"
            self.licenses_value = "license_detections"
        else:
            self.licenses_value = "licenses"
            self.copyright_value = "value"
        self.tool = headers['tool_name']
        self.tool_version = headers['tool_version']

    def read(self):
        with open(self.file_name) as fp:
            self.json_data = json.load(fp)
            self.check_file_format()
            self.files = self.json_data['files']

        files = []

        for f in self.files:
            if not f['type'] == "file":
                continue

            _file = {}

            # path
            _file['path'] = f.get('path', '')
            _file['sha1'] = f.get('sha1', '')
            _file['md5'] = f.get('md5', '')
            _file['sha256'] = f.get('sha256', '')

            # copyright
            _file['copyrights'] = []
            for c in f['copyrights']:
                # print(" c: " + str(c) + "  reading: " + self.copyright_value)
                _file['copyrights'].append(c[self.copyright_value])

            # license
            _file['license'] = {}
            matches = []
            for le in f[self.licenses_value]:
                vers = self.scancode_format[:3] 
                if vers == "3.0" or vers == "3.2" or vers == "4.0":
                    for match in le['matches']:
                        matches.append({
                            "key": match['license_expression'],
                            "text": match['matched_text']
                        })
                else:
                    if 'key' in le and 'matched_text' in le:
                        matches.append({
                            "key": le['key'],
                            "text": le['matched_text']
                        })

            if self.scancode_format[:3] == "3.0":
                if f['detected_license_expression']:
                    _file['license']['expressions'] = [f['detected_license_expression']]
                else:
                    _file['license']['expressions'] = []
            elif self.scancode_format[:3] == "3.2":
                if f['detected_license_expression']:
                    _file['license']['expressions'] = [f['detected_license_expression']]
                else:
                    _file['license']['expressions'] = []
            elif self.scancode_format[:3] == "4.0":
                if f['detected_license_expression']:
                    _file['license']['expressions'] = [f['detected_license_expression']]
                else:
                    _file['license']['expressions'] = []
            else:
                _file['license']['expressions'] = f['license_expressions']
            _file['license']['matches'] = matches

            files.append(_file)

        meta = {
            "scanner": {
                "tool_name": self.tool,
                "tool_version": self.tool_version,
                "tool_output_format": self.scancode_format
            }
        }

        return {
            "files": files,
            "meta": meta
        }



# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

def summarize_license(license_list):
    l_list = []
    for le in license_list:
        l_list.append(f'( {le} )') # noqa: E201, E202
    return " AND ".join(l_list)



# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import os

scarfer_version = "0.5.7"
scarfer_name = "scarfer"

TOP_DIR = os.path.dirname(os.path.realpath(__file__))
VAR_DIR = os.path.join(TOP_DIR, "var")
DEFAULT_FILE_EXCLUDE_FILE = os.path.join(VAR_DIR, "default-exclude-files.txt")



# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from scarfer.scan_interface import ScanReportReader
from scarfer.scan_interface import ScanReportException

class TestScanReader(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestScanReader, self).__init__(*args, **kwargs)

    def test_bad_reader(self):
        with self.assertRaises(ScanReportException):
            reader = ScanReportReader("bad-path")
            reader.read()


if __name__ == '__main__':
    unittest.main()
